@@ -17,7 +17,7 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
   delta<-(cause!=cens.code)
   if (length(cause)!=length(time)) stop("cause and time not of same length\n"); 
   formula <- attr(cif,"Formula")
-  ldata <- aalen.des2(formula(delete.response(terms(formula))),data=data,model="aalen")
+  ldata <- timereg::aalen.des2(formula(delete.response(terms(formula))),data=data,model="aalen")
   X<-ldata$X;  Z<-ldata$Z;  
 
   antpers<-nrow(X); 
@@ -36,14 +36,14 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
   if ((cause1[1]!=cause2[1])) {
     if (is.null(cif2)==TRUE) stop("Must provide marginal model for both causes"); 
     formula2<-attr(cif2,"Formula"); 
-    ldata2 <- aalen.des2(formula(delete.response(terms(formula2))),data=data,model="aalen");
+    ldata2 <- timereg::aalen.des2(formula(delete.response(terms(formula2))),data=data,model="aalen");
     X2<-ldata2$X; Z2<-ldata$Z;  
     if (is.null(Z2)==TRUE) {npar2<-TRUE; semi2<-0;}  else {Z2<-as.matrix(Z2); npar2<-FALSE; semi2<-1;}
     if (npar2==TRUE) {Z2<-matrix(0,antpers,1); pg2<-1; fixed2<-0;} else {fixed2<-1;pg2<-ncol(Z2);} 
     px2<-ncol(X2);  
     est2<-cif2$cum; 
     if (semi2==1) gamma2<-cif2$gamma  else gamma2<-0; 
-    est2<-Cpred(est2,times,strict=FALSE);
+    est2<-cpred(est2,times);
   } else { 
     X2<-matrix(0,1,1); Z2<-matrix(0,1,1); pg2<-1; px2<-1;  semi2<-0; est2<-matrix(0,1,2); gamma2<-0; 
     npar2<-FALSE; 
@@ -53,11 +53,11 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
 ### For truncation 
   if (is.null(entry)) entry.call <- NULL else entry.call <- 0
   if (is.null(entry)) entry <- rep(0,antpers); 
-  cum1<-Cpred(rbind(rep(0,px+1),cif$cum),entry)[,-1];
+  cum1<-cpred(rbind(rep(0,px+1),cif$cum),entry)[,-1];
   if (cif.model==1) cif1entry  <-  1-exp(-apply(X*cum1,1,sum)- (Z %*% gamma )*entry)
   else if (cif.model==2) cif1entry  <-  1-exp(-apply(X*cum1,1,sum)*exp(Z %*% gamma ))
   if ((model!="COR") && (cause1[1]!=cause2[1])) {
-    cum2<-Cpred(rbind(rep(0,px2+1),cif2$cum),entry)[,-1];
+    cum2<-cpred(rbind(rep(0,px2+1),cif2$cum),entry)[,-1];
     if (cif.model==1) cif2entry  <-  1-exp(-apply(X2*cum2,1,sum)- (Z2 %*% gamma2 )*entry)
     else if (cif.model==2) cif2entry  <-  1-exp(-apply(X2*cum2,1,sum)*exp(Z2 %*% gamma2 ))
   } else {cum2 <- cum1; cif2entry <- cif1entry;}
@@ -75,40 +75,37 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
         ud.cens<-survival::survfit(Surv(time,cause==cens.code)~+1); 
         Gfit<-cbind(ud.cens$time,ud.cens$surv)
         Gfit<-rbind(c(0,1),Gfit); 
-        Gcx<-Cpred(Gfit,time)[,2];
-        if (is.null(entry.call)==FALSE) Gcxe<-Cpred(Gfit,entry)[,2];
+        Gcx<-cpred(Gfit,time)[,2];
+        if (is.null(entry.call)==FALSE) Gcxe<-cpred(Gfit,entry)[,2];
         Gcx <- Gcx/Gcxe; 
-###  Gctimes<-Cpred(Gfit,times)[,2];
         Gctimes<- Gcx ## }}}
       } else if (cens.model=="cox") { ## {{{
         if (npar==TRUE) XZ<-X[,-1] else XZ<-cbind(X,Z)[,-1];
-        ud.cens<-cox.aalen(Surv(time,cause==cens.code)~prop(XZ),n.sim=0,robust=0);
-        Gcx<-Cpred(ud.cens$cum,time)[,2];
+        ud.cens<-timereg::cox.aalen(Surv(time,cause==cens.code)~prop(XZ),n.sim=0,robust=0);
+        Gcx<-cpred(ud.cens$cum,time)[,2];
         RR<-exp(XZ %*% ud.cens$gamma)
         Gcx<-exp(-Gcx*RR)
         Gfit<-rbind(c(0,1),cbind(time,Gcx)); 
         if (is.null(entry.call)==FALSE) {
-          Gcxe<-Cpred(Gfit,entry)[,2];
+          Gcxe<-cpred(Gfit,entry)[,2];
           Gcxe<-exp(-Gcxe*RR)
         }
         Gcx <- Gcx/Gcxe; 
-###  Gctimes<-Cpred(Gfit,times)[,2];
         Gctimes<- Gcx  ## }}}
       } else if (cens.model=="aalen") {  ## {{{
         if (npar==TRUE) XZ<-X[,-1] else XZ<-cbind(X,Z)[,-1];
-        ud.cens<-aalen(Surv(time,cause==cens.code)~XZ,n.sim=0,robust=0);
-        Gcx<-Cpred(ud.cens$cum,time)[,-1];
+        ud.cens<-timereg::aalen(Surv(time,cause==cens.code)~XZ,n.sim=0,robust=0);
+        Gcx<-cpred(ud.cens$cum,time)[,-1];
         XZ<-cbind(1,XZ); 
         Gcx<-exp(-apply(Gcx*XZ,1,sum))
         Gcx[Gcx>1]<-1; Gcx[Gcx<0]<-0
         Gfit<-rbind(c(0,1),cbind(time,Gcx)); 
         if (is.null(entry.call)==FALSE) {
-          Gcxe<-Cpred(Gfit,entry)[,-1];
+          Gcxe<-cpred(Gfit,entry)[,-1];
           Gcxe<-exp(-apply(Gcxe*XZ,1,sum))
           Gcxe[Gcxe>1]<-1; Gcxe[Gcxe<0]<-0
         }
         Gcx <- Gcx/Gcxe; 
-###  Gctimes<-Cpred(Gfit,times)[,2];
         Gctimes<- Gcx  ## }}}
       }
     } else { Gcx <- cens.weight; Gctimes <- cens.weight;} 
@@ -152,7 +149,6 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
      dimpar <- ncol(theta.des); 
  
      if (nrow(theta.des)!=ncol(random.design)) stop("nrow(theta.des)!= ncol(random.design)"); 
-###     score.method <- "nlminb"; ### force nlminb because derivatives are not working
  } else random.design <- matrix(0,1,1); 
 
   if ( (!is.null(par.func)) && is.null(dimpar) ) 
@@ -183,7 +179,7 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
   if (is.null(weights)==TRUE) weights <- rep(1,antpers); 
   ## }}}
 
-###  ## {{{ function and derivative
+   ###  ## {{{ function and derivative
   if (is.null(par.func)==FALSE)
   {
      flex.func<-1; # use flexible design
@@ -192,8 +188,6 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
 	  score.method <- "nlminb"
 	  dpar.func <- par.func
   }
-###  if (score.method=="nr) 
-###  cat("Score.method set to nlminb for flexible modelling \n"); 
   } ## }}}
 
   Zgamma <-  c(Z %*% gamma); 
@@ -202,9 +196,6 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
   dep.model <- switch(model,COR=1,RR=2,OR=3,RANCIF=4,ARANCIF=5)
   if (dep.model==0) stop("model must be COR, OR, RR, RANCIF, ARANCIF \n"); 
   if (dep.model<=4) rvdes <- matrix(0,1,1); 
-###  if (dep.model==4 & !is.null(cif2) ) dep.model <- 6 ## two cause random cif
-###  if (dep.model==6) stop("Different causes  under development \n"); 
-
   
   obj <- function(par) 
     { ## {{{
@@ -226,7 +217,6 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
 			  stop("dparfunc must return matrix n x dimpar when called on dparfunc(par,t,theta.des)"); 
 		  DXtheta[s,,] <- Dttheta
 	  }
-###	  print(dim(Xtheta)); print(dim(DXtheta))
     }
 
       outl<-.Call("cor", ## {{{
@@ -271,12 +261,6 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
 	  cat("hess:"); print(hess); 
         }## }}}
         delta <- hessi %*% out$score 
-	### for test purposes
-###     oout <- 0;  ### output control for obj
-###     score1 <- numDeriv::jacobian(obj,p)
-###	hess1 <-  numDeriv::hessian(obj,p)
-###	print(score1)
-###	print(hess1)
 	## do not update last iteration 
 	if (i<Nit) p <- p-delta* step
 	if (is.nan(sum(out$score))) break; 
@@ -296,10 +280,8 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
           cat("theta:");print(c(p))
           cat("score:");print(c(out$score)); 
 	  cat("hess:"); print(hess); 
-###	  oout <- 0; hess1 <-  numDeriv::hessian(obj,p); print(hess1)
     }## }}}
     if (!is.na(sum(hess))) hessi <- lava::Inverse(out$Dscore) else hessi <- diag(nrow(hess))
-###    score1 <- numDeriv::jacobian(obj,p)
     score1 <- score; 
     ## }}}
   } else if (score.method=="nlminb") { ## {{{ nlminb optimizer
@@ -363,6 +345,8 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
   return(ud);
 } ## }}}
 
+##' Cross-odds-ratio, OR or RR risk regression for competing risks
+##'
 ##' Fits a parametric model for the log-cross-odds-ratio for the 
 ##' predictive effect of for the cumulative incidence curves for \eqn{T_1} 
 ##' experiencing cause i given that \eqn{T_2} has experienced a cause k :
@@ -422,9 +406,8 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
 ##' \eqn{h()} may be specified as an R-function of the parameters, 
 ##' see example below, but the default is that it is simply \eqn{\theta^T z}.
 ##'
-##' @title Cross-odds-ratio, OR or RR risk regression for competing risks
 ##' @aliases or.cif rr.cif
-##' @param cif a model object from the comp.risk function with the 
+##' @param cif a model object from the timereg::comp.risk function with the
 ##' marginal cumulative incidence of cause1, i.e., the event of interest, and whose
 ##' odds the comparision is compared to the conditional odds given cause2
 ##' @param data a data.frame with the variables.
@@ -473,18 +456,18 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
 ##' theta.des <- model.matrix(~-1+factor(zyg))
 ##' 
 ##' times=seq(0.05,1,by=0.05) # to speed up computations use only these time-points
-##' add<-comp.risk(Event(time,cause)~+1+cluster(id),data=multcif,cause=1,
+##' add <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),data=multcif,cause=1,
 ##'                n.sim=0,times=times,model="fg",max.clust=NULL)
-##' add2<-comp.risk(Event(time,cause)~+1+cluster(id),data=multcif,cause=2,
+##' add2 <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),data=multcif,cause=2,
 ##'                n.sim=0,times=times,model="fg",max.clust=NULL)
 ##' 
-##' out1<-cor.cif(add,data=multcif,cause1=1,cause2=1)
+##' out1 <- cor.cif(add,data=multcif,cause1=1,cause2=1)
 ##' summary(out1)
 ##' 
-##' out2<-cor.cif(add,data=multcif,cause1=1,cause2=1,theta.des=theta.des)
+##' out2 <- cor.cif(add,data=multcif,cause1=1,cause2=1,theta.des=theta.des)
 ##' summary(out2)
 ##' 
-##' ##out3<-cor.cif(add,data=multcif,cause1=1,cause2=2,cif2=add2)
+##' ##out3 <- cor.cif(add,data=multcif,cause1=1,cause2=2,cif2=add2)
 ##' ##summary(out3)
 ##' ###########################################################
 ##' # investigating further models using parfunc and dparfunc
@@ -496,7 +479,7 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
 ##' table(prt$status)
 ##' 
 ##' times <- seq(40,100,by=10)
-##' cifmod <- comp.risk(Event(time,cause)~+1+cluster(id),data=prt,
+##' cifmod <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),data=prt,
 ##'                     cause=1,n.sim=0,
 ##'                     times=times,conservative=1,max.clust=NULL,model="fg")
 ##' theta.des <- model.matrix(~-1+factor(zyg),data=prt)
@@ -665,7 +648,7 @@ or.cif<-function(cif,data,cause=NULL,cif2=NULL,times=NULL,
 ##' \donttest{ ## Reduce Ex.Timings
 ##'  d <- simnordic.random(5000,delayed=TRUE,cordz=0.5,cormz=2,lam0=0.3,country=TRUE)
 ##'  times <- seq(50,90,by=10)
-##'  add1<-comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
+##'  add1 <- timereg::comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
 ##'  times=times,cause=1,max.clust=NULL)
 ##' 
 ##'  ### making group indidcator 
@@ -682,12 +665,12 @@ or.cif<-function(cif,data,cause=NULL,cif2=NULL,times=NULL,
 ##' ##### 2 different causes
 ##' #########################################
 ##' 
-##'  add2<-comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
+##'  add2 <- timereg::comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
 ##'                   times=times,cause=2,max.clust=NULL)
-##'  out3<-random.cif(add1,data=d,cause1=1,cause2=2,cif2=add2,sym=1,same.cens=TRUE)
+##'  out3 <- random.cif(add1,data=d,cause1=1,cause2=2,cif2=add2,sym=1,same.cens=TRUE)
 ##'  summary(out3) ## negative dependence
 ##' 
-##'  out4<-random.cif(add1,data=d,cause1=1,cause2=2,cif2=add2,theta.des=mm,sym=1,same.cens=TRUE)
+##'  out4 <- random.cif(add1,data=d,cause1=1,cause2=2,cif2=add2,theta.des=mm,sym=1,same.cens=TRUE)
 ##'  summary(out4) ## negative dependence
 ##' }
 ##' @keywords survival
@@ -706,6 +689,8 @@ random.cif<-function(cif,data,cause=NULL,cif2=NULL,
   fit
 } ## }}}
 
+##'Additive Random effects model for competing risks data for polygenetic modelling
+##' 
 ##'Fits a random effects  model describing the dependence in the cumulative 
 ##'incidence curves for subjects within a cluster.  Given the gamma distributed
 ##'random effects it is assumed that the cumulative incidence curves 
@@ -739,10 +724,9 @@ random.cif<-function(cif,data,cause=NULL,cif2=NULL,
 ##'\lambda = pard \theta 
 ##'}
 ##'
-##' @title Additive Random effects model for competing risks data for polygenetic modelling
 ##'
 ##'
-##' @param cif a model object from the comp.risk function with the 
+##' @param cif a model object from the timereg::comp.risk function with the
 ##' marginal cumulative incidence of cause2, i.e., the event that is conditioned on, and whose
 ##' odds the comparision is made with respect to
 ##' @param data a data.frame with the variables.
@@ -794,7 +778,7 @@ random.cif<-function(cif,data,cause=NULL,cif2=NULL,
 ##'  d <- simnordic.random(5000,delayed=TRUE,
 ##'        cordz=1.0,cormz=2,lam0=0.3,country=TRUE)
 ##'  times <- seq(50,90,by=10)
-##'  addm<-comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
+##'  addm <- timereg::comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
 ##'  times=times,cause=1,max.clust=NULL)
 ##' 
 ##'  ### making group indidcator 
@@ -906,19 +890,19 @@ print.summary.cor <- function(x,digits=3,...)
 ##' @author Thomas Scheike
 ##' @keywords survival
 ##' @examples
-##' library("timereg")
-##' data("multcif",package="mets") # simulated data 
-##' multcif$cause[multcif$cause==0] <- 2
-##' 
-##' times=seq(0.1,3,by=0.1) # to speed up computations use only these time-points
-##' add<-comp.risk(Event(time,cause)~+1+cluster(id),data=multcif,
-##'                n.sim=0,times=times,cause=1)
+##' ## library("timereg")
+##' ## data("multcif",package="mets") # simulated data 
+##' ## multcif$cause[multcif$cause==0] <- 2
+##' ##  
+##' ## times=seq(0.1,3,by=0.1) # to speed up computations use only these time-points
+##' ## add <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),
+##' ##                           data=multcif,n.sim=0,times=times,cause=1)
 ##' ###
-##' out1<-cor.cif(add,data=multcif,cause1=1,cause2=1,theta=log(2+1))
-##' summary(out1)
-##' 
-##' pad <- predict(add,X=1,se=0,uniform=0)
-##' summary(out1,marg.cif=pad)
+##' ## out1<-cor.cif(add,data=multcif,cause1=1,cause2=1,theta=log(2+1))
+##' ## summary(out1)
+##' ## 
+##' ## pad <- predict(add,X=1,se=0,uniform=0)
+##' ## summary(out1,marg.cif=pad)
 ##' @method summary cor
 ##' @export
 summary.cor <- function(object,marg.cif=NULL,marg.cif2=NULL,digits=3,...) { ## {{{
@@ -985,7 +969,6 @@ summary.cor <- function(object,marg.cif=NULL,marg.cif2=NULL,digits=3,...) { ## {
     } ## }}} 
     outcase <- cbind(time,casewise,caselower,caseup)
     outconc <- cbind(time,concordance,conclower,concup)
-###    rownames(outcase) <- rownames(outconc)  <-  rownames(coefs)
     colnames(outcase) <- c("time","casewise concordance","2.5 %","97.5%")
     colnames(outconc) <- c("time","concordance","2.5 %","97.5%")
 	  if (length(thetav)==1) {
@@ -1144,7 +1127,6 @@ concordanceCor <- function(object,cif1,cif2=NULL,messages=TRUE,model=NULL,coefs=
 
     out[[k]] <- list(concordance=outconc,casewise=outcase)
     names(out)[k] <- rownames(coefs)[k]
-###k <- k+1
   }
 
   return(out)
@@ -1245,7 +1227,6 @@ coef.randomcif<- function (object, digits = 3, ...)
   colnames(res) <- c("Coef.", "SE", "z", "P-val", "Cross odds ratio", "SE")
   if (!is.null(object$thetanames)) rownames(res)<-object$thetanames
 
-###  prmatrix(signif(res, digits))
   return(res)
 } ## }}}
 
@@ -1304,26 +1285,6 @@ coef.randomcifrv<- function (object, digits = 3, ...)
    if (!is.null(object$thetanames)) rownames(res)<-object$thetanames
    prmatrix(signif(res, digits))
 
-###    cat("\n\n Random effect variances for gamma random effects \n\n")
-###    varpar <- theta/sum(theta)^2 
-###    res <- as.matrix(varpar); 
-###    if (elog==0)  { var.theta <-   object$var.theta; 
-###                    df <- 0*var.theta; 
-###                    for (i in 1:nrow(var.theta))
-###                    df[i,] <- -theta[i]*2*theta; 
-###		    diag(df) <- diag(df)+sum(theta)^2
-###		    df <- df/sum(theta)^4
-###		    var.varpar <- df %*% var.theta %*% df
-###                  }
-###    if (elog==1)  { 
-###	            var.theta <-   object$var.theta; 
-###                    var.varpar <- var.theta
-###                  }
-###    res <- cbind(res,diag(var.varpar)^.5)  
-###    colnames(res) <- c("variance","SE")
-###    if (is.null((rownames(res))) == TRUE) rownames(res) <- rep(" ", nrow(res))
-###    prmatrix(signif(res, digits))
-
 } ## }}}
 
 ##' @export
